""" Survey

Some code taken from ATTopic in ATContentTypes.

Licensed under the ZPL.

The rest:
(C) 2004 by Dan Keshet (also under the ZPL)

"""
from Products.ATContentTypes.types.ATContentType import ATCTOrderedFolder, updateActions
from Products.ATContentTypes.types.schemata import ATContentTypeSchema
from Products.Archetypes.public import registerType
from Permissions import *
from AccessControl import ClassSecurityInfo

from fields import FieldRegistry
schema = ATContentTypeSchema.copy()	

class Survey(ATCTOrderedFolder):
	"""A type for making user-creatable surveys"""
	schema = schema
	content_icon   = 'PILFERED_ICON.png'
	meta_type      = 'Survey'
	archetype_name = 'Survey'
	use_folder_tabs = 0
	filter_content_types  = 1
	allowed_content_types = ()	
	security = ClassSecurityInfo()

	def __init__(self, id, **kwargs):
		self._users = []
		ATCTOrderedFolder.__init__(self,id, **kwargs)
	
	security.declareProtected('addField', EDIT_CONTENT_PERMISSION)
	def addField(self, field_type):
		"""Add a new question field.
		"""
		newid = self.generateUniqueId(field_type)
		ft    = FieldRegistry.get(field_type,None)
		if ft is None:
			raise ValueError, "%s is not a valid field type" % field_type
		field = ft(newid)
		self._setObject( newid, field )
		
	def getUniqueId(self, username):
		"""Returns a different id for each username, so that we can 
		decide whether to keep track of who answered what or not"""
		try:
			l = self._users.index(username)
		except (ValueError):
			l = len(self._users)
			self._users.append(username)
		return l
	
	def listFieldTypes(self):
		"""List available field types"""
		return FieldRegistry.listTypes()
		
	def listFields(self):
		"""List current fields"""
		return self.objectValues()
	
	actions = updateActions(ATCTOrderedFolder,
		(
		{
		'id'          : 'view',
		'name'        : 'View',
		'action'      : 'string:${folder_url}/',
		'permissions' : (VIEW_CONTENT_PERMISSION,)
		},
		{
		'id'          : 'edit',
		'name'        : 'Edit',
		'action'      : 'string:${object_url}/base_edit',
		'permissions' : (EDIT_CONTENT_PERMISSION,)
		},
		{
		'id'          : 'questions',
		'name'        : 'Questions',
		'action'      : 'string:${folder_url}/survey_questions',
		'permissions' : (EDIT_CONTENT_PERMISSION,)
		},
		{
		'id'          : 'results',
		'name'        : 'Results',
		'action'      : 'string:${folder_url}/survey_results',
		'permissions' : (VIEW_RESULTS,)
		},
		{
		'id'          : 'take',
		'name'        : 'Take Survey',
		'action'      : 'string:${folder_url}/survey_take',
		'permissions' : (TAKE_SURVEY,)
		}
	))

registerType(Survey)