from Products.Archetypes.public import BaseContentMixin, ReadOnlyStorage
from Products.Archetypes.Renderer import renderer
from Products.Survey.fields.schemata import BaseSurveyFieldSchema, BaseSurveyItemsFieldSchema
from Products.Survey.Permissions import *
from AccessControl import ClassSecurityInfo
from BTrees.IOBTree import IOBTree
from types import IntType, StringType, ListType, TupleType

class BaseSurveyField(BaseContentMixin):
	"""A Basic Field"""
	__implements__ = BaseContentMixin.__implements__
	security = ClassSecurityInfo()
	schema = BaseSurveyFieldSchema
	
	meta_type = 'BaseSurveyField'
	archetype_name = 'Base Survey Field'
	typeDescription= ''
	typeDescMsgId  = ''
	fieldType = ''
	widgetType=''
	global_allow = 0
	validators = []

	actions = ({
		'id'          : 'long_results',
		'name'        : 'Results',
		'action'      : 'string:${object_url}/field_results',
		'permissions' : (VIEW_CONTENT_PERMISSION,)
		},)
	
	def __init__(self, id, **kwargs):
		self._storage = IOBTree()
		self.field = self.fieldType(storage=ReadOnlyStorage())
		BaseContentMixin.__init__(self, id, **kwargs)

	security.declareProtected(EDIT_CONTENT_PERMISSION, 'edit')
	def edit(self, **kwargs):
		"""Modify our internal fields, then use them to create the resulting Field
		"""
		BaseContentMixin.update(self,**kwargs)
		fieldkws = widgetkws = {}
		for field in self.Schema().filterFields(buildingBlockFor='field'):
			name = field.getName()
			value = self[name]
			#This is a workaround for fields where get${Field} already has a meaning
			if name.startswith('dummy_'):
				name = name[6:]
			if value is not None:
				fieldkws[name] = value
		for field in self.Schema().filterFields(buildingBlockFor='widget'):
			name = field.getName()
			if kwargs.has_key(name):
				widgetkws[name] = kwargs[name]
		widget = self.widgetType(**widgetkws)
		self.field = self.fieldType(widget=widget,
									storage=ReadOnlyStorage(),
									write_permission = TAKE_SURVEY,
									**fieldkws)

	def accessor(self,uid=None):
		"""Gets the results from one particular survey, or
		default if none specified."""
		if isinstance(uid,StringType):
			uid = int(uid)
		if isinstance(uid,IntType):
			return self._storage.get(uid, self.field.default)
		return self.field.default

	security.declareProtected(TAKE_SURVEY, 'render')
	def render(self, mode='edit',uid=None):
		"""Render question."""
		return renderer.render(self.getId(), #fieldName
							mode,  #mode
							self.field.widget, #widget
							self, #instance
							self.field, #field
							lambda: self.accessor(uid)) #accessor

	security.declareProtected(TAKE_SURVEY, 'addAnswer')
	def addAnswer(self, uid, answer):
		"""store an answer to this question of the survey
		uid is a unique identifier that identifies a survey 'instance'
		The Survey itself is in charge of (optionally) keeping track of 
		which user corresponds to which uid, and of varying uids in case
		they shouldn't be matched to users.
		"""
		s = self._storage
		if s.has_key(uid):
			del(s[uid])
		s.insert(uid,answer)
	
	security.declareProtected(VIEW_RESULTS, 'getResults')
	def getResults(self):
		"""Gets the results from all surveys"""
		return self._storage.items()		
			
class BaseSurveyItemsField(BaseSurveyField):
	"""A Basic Field that picks between listed items"""
	__implements__ = BaseContentMixin.__implements__
	security = ClassSecurityInfo()
	schema = BaseSurveyItemsFieldSchema 
	meta_type = 'BaseSurveyItemsField'
	archetype_name = 'Base Survey Items Field'
	typeDescription= ''
	typeDescMsgId  = ''
	global_allow = 0	
	
	security.declarePublic('getResults')
	def getResults(self):
		"""Gets the results from all surveys"""
		values = list(self._storage.values())
		return [( item, values.count(item) ) for item in self.field.Vocabulary()]