from Products.Archetypes.public import ReadOnlyStorage
from AccessControl import ClassSecurityInfo
from Products.Survey.fields.BaseSurveyField import BaseSurveyField, BaseSurveyItemsField
from Products.Survey.fields import registerField
from Products.Survey.Permissions import *
from Products.Survey.fields.schemata import LabelSchema
from BTrees.IIBTree import IIBTree
from types import ListType, TupleType
from Products.Archetypes.public import *

class NumberField(BaseSurveyField):
	"""A numeric field"""
	meta_type	  = 'NumericField'
	archetype_name = 'Numeric Field'
	shortDesc	  = 'numeric'
	validators	 = 'isInt'
	fieldType = IntegerField
	widgetType = IntegerWidget
	security = ClassSecurityInfo()
	
	def __init__(self, id, **kwargs):
		self._storage = IIBTree()
		self.field = self.fieldType(storage=ReadOnlyStorage())
		BaseContentMixin.__init__(self, id, **kwargs)
	
	security.declareProtected(TAKE_SURVEY, 'addAnswer')
	def addAnswer(self, uid, answer):
		"""Normalize answer into an int before inserting"""
		try:
			BaseSurveyField.addAnswer(self, uid, int(answer))
		except ValueError:
			pass
	
	def getStats(self):
		vals = list(self._storage.values())
		if len(vals) > 0:
			mini = min(vals)
			maxi = max(vals)
		else:
			mini = None
			maxi = None
		return {'min': min(vals),
	            'max': max(vals)}
	
class DateAndTimeField(BaseSurveyField):
	"""A date/time field"""
	meta_type	  = 'DateAndTimeField'
	archetype_name = 'Date Time Field'
	shortDesc	  = 'date/time'
	fieldType = DateTimeField
	widgetType = CalendarWidget
			
class ListField(BaseSurveyField):
	"""A list field"""
	meta_type	  = 'ListField'
	archetype_name = 'List Field'
	shortDesc	  = 'list'
	fieldType = LinesField
	widgetType = LinesWidget
	
class ShortAnswerField(BaseSurveyField):
	"""A simple short answer field"""
	meta_type	  = 'ShortAnswerField'
	archetype_name = 'Survey Short Answer Field'
	shortDesc	  = 'short answer'
	fieldType = StringField
	widgetType = StringWidget
	
class LabelField(BaseSurveyField):
	"""A simple short answer field"""
	schema = LabelSchema
	meta_type	  = 'LabelField'
	archetype_name = 'Label Field'
	shortDesc	  = 'label'
	fieldType = StringField
	widgetType = LabelWidget

class LongAnswerField(BaseSurveyField):
	"""A simple long answer field"""
	meta_type	  = 'LongAnswerField'
	archetype_name = 'Survey Long Answer Field'
	shortDesc	  = 'long answer'
	fieldType = TextField
	widgetType = TextAreaWidget

class PickOneField(BaseSurveyItemsField):
	"""A pick-one field"""
	meta_type	  = 'PickOneField'
	archetype_name = 'Pick One Field'
	shortDesc	  = 'pick one'
	fieldType = StringField
	widgetType = SelectionWidget

class PickManyField(BaseSurveyItemsField):
	"""A pick-many field"""
	meta_type	  = 'PickManyField'
	archetype_name = 'Pick Many Field'
	shortDesc	  = 'pick many'
	fieldType = StringField
	widgetType = MultiSelectionWidget
	
	security = ClassSecurityInfo()

	security.declareProtected(VIEW_RESULTS, 'getResults')
	def getResults(self):
		"""Gets the results from all surveys"""
		results = {}
		for val_list in self._storage.values():
			for value in val_list:
				results[value] = results.get(value,0) + 1
		return [( item, results.get(item,0) ) for item in self.getItems()]

class YesNoField(BaseSurveyField):
	"""A boolean field"""
	meta_type	  = 'YesNoField'
	archetype_name = 'Yes No Field'
	shortDesc	  = 'yes/no'
	fieldType = BooleanField
	widgetType = BooleanWidget
	
	security = ClassSecurityInfo()
	
	def __init__(self, id, **kwargs):
		self._storage = IIBTree()
		self.field = self.fieldType(storage=ReadOnlyStorage())
		BaseContentMixin.__init__(self, id, **kwargs)
	
	security.declareProtected(TAKE_SURVEY, 'addAnswer')
	def addAnswer(self, uid, answer):
		"""Clean up answer before inserting 
		"""
		if not answer:
			answer = 0
		else:
			answer = 1
		BaseSurveyField.addAnswer(self, uid, answer)

registerField(LongAnswerField)
registerField(ShortAnswerField)
registerField(LabelField)
registerField(PickOneField)
registerField(PickManyField)
registerField(YesNoField)
registerField(NumberField)
registerField(DateAndTimeField)
registerField(ListField)